/***************************************************************************
                               calias.h
                             -------------------
    begin                : Thu Jul 25 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CALIAS_H
#define CALIAS_H

// local includes
#include "caliasitem.h"

// forward declarations
class QStringList;

/** Class handling one command alias
  * @author Kmud Developer Team
  */

class CAlias : public CAliasItem {
public:
	/**
	  * Constructs an empty and disabled alias.
	  */
	CAlias();

	/**
	  * Constructs a disabled alias.
	  * @param name the name of the alias. Any string allowed.
	  * @param alias the string that can be used as an alias for the command.
	  *              This is usually a short mnemonic fro the much longer command.
	  * @param command the string that will replace the alias in the user input string.
	  *                It may contain any characters other than %.
	  *                % followd by a digit 0-9 is treated as a variable which will be replaced
	  *                by the words that appear in the input after the alias.
	  */
	CAlias(const QString& name, const QString& alias, const QString& command);


	CAlias(const CAlias& alias) : CAliasItem(alias) {
		if (&alias == this) return;
		copy(alias);
	}

	virtual ~CAlias();

	inline CAlias& operator=(const CAlias& alias) {
		if (&alias != this)
			copy(alias);
		return *this;
	}

	inline Type type() const { return Alias; }
	
	/**
	  * Sets the alias of the Alias item.
	  * @param alias the string that can be used as an alias fro the command.
	  *              This is usually a short mnemonic fro the much longer command.
	  */
	void setAlias(const QString& alias) { m_alias = alias; }

	/**
	  * Sets the command of the Alias item.
	  * @param command the string that will replace the alias in the user input string.
	  *                It may contain any characters other than %.
	  *                % followd by a digit 0-9 is treated as a variable which will be replaced
	  *                by the words that appear in the input after the alias.
	  */
	void setCommand(const QString& command);

	/**
	  * Returns the Aliasitem's alias.
	  */
	QString alias() const { return m_alias; }

	/**
	  * Returns the Aliasitem's command.
	  */
	QString command() const { return m_alias; }

	/**
	  * Read the item's content from the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	 bool readFromDOM(const QDomElement& dom);

	/**
	  * Write the item's content to the given QDomElement.
	  * @return true if reading succeeded, false otherwise.
	  */
	bool writeToDOM(QDomElement& dom);

	/**
	  * Returns a string constructed of the Item's command where all variabled have been
	  * replaced by the contents of the input string list.
	  * If the first item of the inout list doesn't match the alias string,
	  * it returns QString::null.
	  * If there were more variables than input items, the remaining variales are removed.
	  * If there were more input items than variabled the remaining inout items are appended,
	  * seperated by a single whitespace.
	  * @param input a list of words.
	  * @return Returns the command string with all variables resolved, or QString::null
	  *         if the input list is empty, the first item of it doesn't match the alias string or
	  *         if the Aliasitem is disabled.
	  */
	QString replace(const QStringList& input);

protected:
	static const uint MAX_ARGS = 9;

	QString m_name;
	QString m_alias;
	QString m_command;
	bool    m_args[MAX_ARGS];
	int     m_argMax;

protected:

	bool resolveCommands(QStringList& inputList);

private:
	inline void copy(const CAlias& alias) {
		CAliasItem::operator=(alias);
		m_alias    = alias.m_alias;
		m_command  = alias.m_command;
		for (uint i = 0; i < MAX_ARGS; ++i)
		{
			m_args[i] = alias.m_args[i];
		}
		m_argMax = alias.m_argMax;
	}

};

#endif

// end of file
