/***************************************************************************
                               cansiparser.cpp
                             -------------------
    begin                : Tue Apr 30 2002
    copyright            : (C) 2002 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cansiparser.h"

// local includes
#include "cansiparser.h"
#include "cansiinterpreter.h"

CANSIParser::CANSIParser() : m_interpreter(0)
{
}

CANSIParser::~CANSIParser()
{
}

void CANSIParser::parse(const QString& input)
{
//	qDebug("CANSIParser::parse(%s)", input.latin1());
	if (input.isEmpty())
		return;

	QString buffer = m_buffer;
	m_buffer = QString::null;
	buffer.append(input);

	if (buffer[0] == ANSI::ESC)
	{
		parseANSI(buffer);
	}
	else
	{
		parseText(buffer);
	}
}

void CANSIParser::parseANSI(const QString& input)
{
//	qDebug("CANSIParser::parseANSI(%s)", input.latin1());
	if (input.isEmpty())
		return;

	if (input.length() == 1)
	{
		m_buffer.append(input);
		return;
	}

	if (input[1] != '[')
	{
		int posOfEsc = input.find(ANSI::ESC, 1);
		if (posOfEsc == -1)
			m_buffer.append(input);
		else
		{
			m_buffer.append(input.left(posOfEsc));
			parse(input.mid(posOfEsc, input.length()));
		}
		return;
	}

	// input begins with ESC[
	// look for end of ANSI sequence
	uint seqEnd = 0;
	for (uint i = 2; i < input.length(); ++i)
	{
		switch (input[i])
		{
			case 'H':
			case 'f':
				seqEnd = i;
				break;
			case 'A':
				seqEnd = i;
				break;
			case 'B':
				seqEnd = i;
				break;
			case 'C':
				seqEnd = i;
				break;
			case 'D':
				seqEnd = i;
				break;
			case 's':
				seqEnd = i;
				break;
			case 'u':
				seqEnd = i;
				break;
			case 'J':
				seqEnd = i;
				break;
			case 'K':
				seqEnd = i;
				break;
			case 'h':
				seqEnd = i;
				break;
			case 'l':
				seqEnd = i;
				break;
			case 'm':
				seqEnd = i;
				parseGraphicsMode(input.mid(2, seqEnd-2));
				break;
			default:
				break;
		}

		if (seqEnd != 0)
			break;
	}

	if(seqEnd == 0) // sequence end not found in this input chunk
		m_buffer = input;
	else if ((seqEnd+1) < input.length())
		parse(input.mid(seqEnd+1, input.length()));
}

void CANSIParser::parseText(const QString& input)
{
//	qDebug("CANSIParser::parseText(%s)", input.latin1());
	if (input.isEmpty())
		return;

	int posOfEsc = input.find(ANSI::ESC);
	if (posOfEsc == -1)
	{
		if (m_interpreter != 0)
			m_interpreter->inputText(input);
	}
	else
	{
		if ((posOfEsc+1) < (int)input.length())
		{
			if (input[posOfEsc+1] == '[')
			{
				if (m_interpreter != 0)
					m_interpreter->inputText(input.left(posOfEsc));

				parseANSI(input.mid(posOfEsc, input.length()));
			}
			else // this is an ESC without [ => no ANSI
			{
				if (m_interpreter != 0)
					m_interpreter->inputText(input.left(posOfEsc+1));

				parse(input.mid((posOfEsc+1), input.length()));
			}
		}
		else if (posOfEsc > 0) // ESC at end, but there is some text
		{
			if (m_interpreter != 0)
				m_interpreter->inputText(input.left((posOfEsc-1)));

			m_buffer.append(input.mid(posOfEsc, input.length()));
		}
		else // ESC is the only input
		{
			m_buffer.append(input);
		}
	}
}

void CANSIParser::parseGraphicsMode(const QString& input)
{
//	qDebug("CANSIParser::parseGraficsMode(%s)", input.latin1());
	if (input.isEmpty())
		return;

	if (m_interpreter == 0)
		return;

	QString command;
	for (uint i = 0; i < input.length(); ++i)
	{
		if (input[i] == ';' || i == (input.length()-1))
		{
			if (input[i] != ';') // last number
				command.append(input[i]);

			switch (command.toInt())
			{
				case 0:
					m_interpreter->inputTextAttribute(ANSI::NORMAL);
					break;
				case 1:
					m_interpreter->inputTextAttribute(ANSI::BOLD);
					break;
				case 2:
					m_interpreter->inputTextAttribute(ANSI::DIM);
					break;
				case 4:
					m_interpreter->inputTextAttribute(ANSI::UNDERLINE);
					break;
				case 5:
					m_interpreter->inputTextAttribute(ANSI::BLINK);
					break;
				case 7:
					m_interpreter->inputTextAttribute(ANSI::REVERSE);
					break;
				case 8:
					m_interpreter->inputTextAttribute(ANSI::HIDDEN);
					break;
				case 30:
					m_interpreter->inputTextColor(ANSI::BLACK);
					break;
				case 31:
					m_interpreter->inputTextColor(ANSI::RED);
					break;
				case 32:
					m_interpreter->inputTextColor(ANSI::GREEN);
					break;
				case 33:
					m_interpreter->inputTextColor(ANSI::YELLOW);
					break;
				case 34:
					m_interpreter->inputTextColor(ANSI::BLUE);
					break;
				case 35:
					m_interpreter->inputTextColor(ANSI::MAGENTA);
					break;
				case 36:
					m_interpreter->inputTextColor(ANSI::CYAN);
					break;
				case 37:
					m_interpreter->inputTextColor(ANSI::WHITE);
					break;
				case 40:
					m_interpreter->inputBackColor(ANSI::BLACK);
					break;
				case 41:
					m_interpreter->inputBackColor(ANSI::RED);
					break;
				case 42:
					m_interpreter->inputBackColor(ANSI::GREEN);
					break;
				case 43:
					m_interpreter->inputBackColor(ANSI::YELLOW);
					break;
				case 44:
					m_interpreter->inputBackColor(ANSI::BLUE);
					break;
				case 45:
					m_interpreter->inputBackColor(ANSI::MAGENTA);
					break;
				case 46:
					m_interpreter->inputBackColor(ANSI::CYAN);
					break;
				case 47:
					m_interpreter->inputBackColor(ANSI::WHITE);
					break;
				default:
					break;
			}
			command = QString::null;
		}
		else
		{
			command.append(input[i]);
		}
	}
}
