/***************************************************************************
                               cmccp.cpp
                             -------------------
    begin                : 06.04.2002
    copyright            : (C) 2002 by Christian Welzel
    email                : gawain@camlann.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmccp.h"

#include <arpa/telnet.h>
#include "kmuddebug.h"


// compress not defined in telnet standard
#define TELOPT_COMPRESS		85
#define TELOPT_COMPRESS2	86


CMCCP::CMCCP() : QObject() {
	m_state     = 0;
	m_compress  = 0;
	m_error     = 0;

	m_stats.in  = 0L;
	m_stats.out = 0L;
	m_stream.zalloc = zlib_alloc;
	m_stream.zfree  = zlib_free;
	m_stream.opaque = NULL;

	m_buffer    = NULL;
	m_bufpos    = 0;
	m_buflen    = 0;

	inflateInit(&m_stream);
}

CMCCP::~CMCCP() {
	inflateEnd(&m_stream);
}

void CMCCP::reset() {
	inflateEnd(&m_stream);
	m_state     = 0;
	m_compress  = 0;
	m_error     = 0;
	m_stats.in = m_stats.out = 0L;
	m_buffer    = NULL;
	m_bufpos    = 0;
	m_buflen    = 0;
	inflateInit(&m_stream);
}

int CMCCP::process(QByteArray &data) {

	if(m_error) return m_error;

	m_stats.in += data.size();

	m_buffer   = new unsigned char [data.size()];
	m_buflen   = data.size();
	m_bufpos   = 0;

	int next_pos   = scan((unsigned char *) data.data(), data.size());

	if(next_pos >= 0) {
	m_compress = 1;
	if(next_pos != m_buflen) // only when we not yet at the end of input buffer
		m_error = decompress((unsigned char *) data.data(), data.size(), next_pos);
	}

	data.assign((char *) m_buffer, m_bufpos); // possible mem-leak! sizeof(m_buffer) is longer then m_bufpos
	m_buffer = NULL; // to prevent accidentally freeing of the mem, now managed by QByteArray

	m_stats.out += data.size();

	kdDebug(DAREA_CORE_TELNET)<<"cmccp::stats::in="<<m_stats.in<<"::out="<<m_stats.out<<endl;

	return m_error;
}

int CMCCP::decompress(unsigned char *in, ulong size, int start_pos) {

	m_stream.next_in   = in+start_pos;
	m_stream.avail_in  = size-start_pos;
	m_stream.next_out  = m_buffer+m_bufpos;
	m_stream.avail_out = m_buflen-m_bufpos;

	int err = inflate(&m_stream, Z_SYNC_FLUSH);

	m_bufpos = m_buflen - m_stream.avail_out;

	if(err) emit(errorSignal(err));

	if(err == Z_OK && m_stream.avail_out == 0) { // zlib needs more output mem
		unsigned char *b, *c = m_buffer, *d;
	
		d = b = new unsigned char [m_buflen + 200];
		while((*d++ = *c++));
		free(m_buffer);
		m_buffer = b;
		m_buflen += 200;
	
		err = decompress(in, size, size-m_stream.avail_in);
   	}

	return err;
}

int CMCCP::scan(unsigned char *in, ulong size) {
	unsigned int i=0, will=0;
	unsigned char c;

	if(m_compress) return 0;

	while(i < size) {

	c = in[i];
	switch(m_state) {
		case 0: // NORMAL MODE
			if     (c == IAC)  m_state = 1;
			break;
		case 1: // IAC
			if     (c == WILL) m_state = 2;
			else if(c == SB)   m_state = 5;
			else               m_state = 0;
			break;
		case 2: // IAC WILL
			if     (c == TELOPT_COMPRESS)  {
				QString t;
				t.sprintf("%c%c%c", IAC, DO, TELOPT_COMPRESS);
				m_bufpos >= 2 ? m_bufpos -= 2 : m_bufpos = 0;
				will = 1;
				kdDebug(DAREA_CORE_TELNET)<<"cmccp::scan()::IAC WILL COMPRESS"<<endl;
				emit(sendToServer(t));
		    }
			else if(c == TELOPT_COMPRESS2) {
				QString t;
				t.sprintf("%c%c%c", IAC, DO, TELOPT_COMPRESS2);
				will = 1;
				m_bufpos >= 2 ? m_bufpos -= 2 : m_bufpos = 0;
				kdDebug(DAREA_CORE_TELNET)<<"cmccp::scan()::IAC WILL COMPRESS2"<<endl;
				emit(sendToServer(t));
			}
			m_state = 0;
			break;
		case 5: // IAC SB
			if     (c == TELOPT_COMPRESS)  m_state = 6;
			else if(c == TELOPT_COMPRESS2) m_state = 8;
			else                    m_state = 0;
			break;
		case 6: // IAC SB COMPRESS
			if     (c == WILL)      m_state = 7;
			else                    m_state = 0;
			break;
		case 7: // IAC SB COMPRESS WILL
			if     (c == SE) { // IAC SB COMPRESS WILL SE
			kdDebug(DAREA_CORE_TELNET)<<"cmccp::scan()::IAC SB COMPRESS WILL SE"<<endl;
			m_bufpos >= 4 ? m_bufpos -= 4 : m_bufpos = 0;
    			return i+1;
			}
			m_state = 0;
			break;
		case 8: // IAC SB COMPRESS2
			if     (c == IAC)       m_state = 9;
			else                    m_state = 0;
			break;
		case 9: // IAC SB COMPRESS2 IAC
			if     (c == SE) { // IAC SB COMPRESS2 IAC SE
			kdDebug(DAREA_CORE_TELNET)<<"cmccp::scan()::IAC SB COMPRESS2 IAC SE"<<endl;
			m_bufpos >= 4 ? m_bufpos -= 4 : m_bufpos = 0;
    			return i+1;
			}
			m_state = 0;
			break;
	} // switch
	if(!will)   m_buffer[m_bufpos++] = in[i];
	++i; will = 0;
	}

	return -1;
}

bool CMCCP::error() const {
	return m_error;
}

bool CMCCP::isCompressing() const{
	return m_compress;
}

const cmccpstats &CMCCP::stats() const {
	return m_stats;
}
