/***************************************************************************
                               kmudcore.cpp
                             -------------------
    begin                : Sun May 20 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qdom.h>
#include <kapp.h>
#include <klibloader.h>
#include <ktrader.h>
#include <kservice.h>

#include <kmudcore.h>

#include <kmuddebug.h>
#include <kmudmainwindow.h>
#include <kmudviewmanager.h>
#include <kmudconfigmanager.h>
#include <kmudview.h>
#include <ctelnet.h>
#include <cmudstream.h>
#include <kmudconnecthistory.h>
#include <domconfig.h>
#include <kmemconfig.h>
#include <kmudfactory.h>
#include <kmudpart.h>

#include "kmudconfigcache.h"
#include "cautologinfilter.h"
#include "caliasfilter.h"


KmudCore::KmudCore()
{
	kdDebug(DAREA_CORE) << "constructor called" << endl;

	m_connectionState = Disconnected;

	kdDebug(DAREA_CORE) << "retrieving KmudConfigManager instance" << endl;
	m_configManager = KmudConfigManager::self();

	kdDebug(DAREA_CORE) << "creating CMudStream" << endl;
	m_mudstream = new CMudStream(this,"KmudCore::m_mudstream");

	kdDebug(DAREA_CORE) << "creating CTelnet" << endl;
	m_telnet = new CTelnet(this,"KmudCore::m_telnet");
	m_mudstream->registerInputFilter(m_telnet);

	kdDebug(DAREA_CORE) << "creating new KmudMainWindow" << endl;
	m_kmud = new KmudMainWindow(this,0L,"KmudMainWindow");
	kapp->setMainWidget(m_kmud);
	m_view = m_kmud->getView();

	connect(m_telnet,SIGNAL(changedEcho(bool)),m_view,SLOT(setLocalEcho(bool)));
	connect(m_telnet,SIGNAL(changedEcho(bool)),m_view,SLOT(setInputEcho(bool)));

	m_mudstream->registerOutputFilter(m_view->filter());

	kdDebug(DAREA_CORE) << "creating PartManager" << endl;
	m_partManager = new KParts::PartManager(m_kmud,"PartManager");
	// When the manager says the active part changes, the builder updates (recreates) the GUI
	connect(m_partManager, SIGNAL(activePartChanged(KParts::Part*)), m_kmud, SLOT(createGUI(KParts::Part*)));

	m_partList.setAutoDelete(false);


	kdDebug(DAREA_CORE) << "creating KmudViewManager" << endl;
	m_viewManager = new KmudViewManager(m_kmud, this, "KmudViewManager" );

	kdDebug(DAREA_CORE) << "creating CAliasFilter" << endl;
	m_aliasFilter = new CAliasFilter(this);
	m_mudstream->registerInputFilter(m_aliasFilter);

	m_autoLoginFilter = new CAutoLoginFilter(this,this,"AutoLoginFilter");
	m_mudstream->registerOutputFilter(m_autoLoginFilter);
	
	// load mapper
	//FIXME : This should be automatic
	loadPart("Kmud/MapperPart");

	m_lastMudPort = 23;

	m_kmud->show();

	kdDebug(DAREA_CORE) << "constructor finished" << endl;
}

KmudCore::~KmudCore()
{
	KmudPart* part;

	kdDebug(DAREA_CORE) << "destructor called" << endl;

	disconnect(m_viewManager, SIGNAL(activePartChanged(KParts::Part*)), m_kmud, SLOT(createGUI(KParts::Part*)));

	while (m_partList.isEmpty()==false)
	{
		part = m_partList.last();
		kdDebug(DAREA_CORE) << "deleting part " << part << endl;
		m_partManager->removePart(part);
		m_partList.remove(part);
		delete part;
		part = 0L;
	}

	delete m_aliasFilter;

	kdDebug(DAREA_CORE) << "destructor finished" << endl;
}

KmudMainWindow* KmudCore::mainWindow()
{
	return m_kmud;
}

KmudViewManager* KmudCore::viewManager()
{
	return m_viewManager;
}

CTelnet* KmudCore::telnetConnection()
{
	return m_telnet;
}

CMudStream* KmudCore::mudStream()
{
	return m_mudstream;
}

bool KmudCore::connectToMud(QString connectStr)
{
	QString charProfile;
	QString mudTitle;
	QString login;
	QString mudHost;
	uint mudPort = 0;

	int index = connectStr.find('/');
	if (index > 0)
	{
		charProfile = connectStr;
		QDomElement charProfileElement = profileAPI()->openProfile(charProfile);
		if (charProfileElement.isNull())
		{
			kdDebug(DAREA_CORE) << "connectToMud: opening profile " << charProfile << " failed.\n";
			return false;
		}
	
		DomConfig* charDomConfig = DomConfig::findGroup("/", charProfileElement);
		if (charDomConfig == 0)
		{
			kdDebug(DAREA_CORE) << "connectToMud: no root config in char profile element!\n";
			return false;
		}
		QString mudProfile = charProfile.left(index);
		QDomElement mudProfileElement = profileAPI()->openProfile(mudProfile);
		DomConfig* mudDomConfig = DomConfig::findGroup("/", mudProfileElement);
		if (mudDomConfig == 0)
		{
			kdDebug(DAREA_CORE) << "connectToMud: no root config in mud profile element!\n";
			delete charDomConfig;
			return false;
		}
		KMemConfig* config = mudDomConfig->config();
		config->setGroup("General");
		// fixme_kevin: use better defaults
		mudTitle = config->readEntry("Name", "localhost");
		mudHost = config->readEntry("Server", "127.0.0.1");
		mudPort = config->readUnsignedNumEntry("Port", 23);
		config->writeEntry("Name", mudTitle);
		config->writeEntry("Server", mudHost);
		config->writeEntry("Port", mudPort);
		mudDomConfig->save();
		profileAPI()->saveProfile(mudProfile);

		config = charDomConfig->config();
		config->setGroup("Login");
		login = config->readEntry("Login", "kevin");
		
		delete charDomConfig;
		delete mudDomConfig;	
	}
	else if (index == -1)
	{
		index = connectStr.find(':');
		if (index == -1)
			return false;

		mudHost = connectStr.left(index);
		mudPort = connectStr.mid(index+1).toUInt();
	}
	else
		return false;

	bool result = connectToMud(mudTitle,login,mudHost,mudPort);
	if (result)
	{
		if (!charProfile.isEmpty())
		{
			setProfile(charProfile);
			KmudConnectHistory::addConnection(charProfile);
		}
		else
		{
			setProfile(QString::null);
			QString entry = QString("%1:%2").arg(m_lastMudHost).arg(m_lastMudPort);
			KmudConnectHistory::addConnection(entry);
		}
	}
	return result;
}

bool KmudCore::connectToMud(QString mudTitle, QString login, QString mudHost, uint mudPort)
{
	int i;
	bool retVal=false;

	m_kmud->getView()->clear();
	m_kmud->setCaption(mudTitle, false);

	m_connectionState = Connecting;
	emit(connectionStateChanged());

	i = m_telnet->connect(mudHost,mudPort,configCache()->connectionTimeoutSeconds);

	switch (i)
	{
		case 0: m_connectionState = Connected;
				emit(connectionStateChanged());
				retVal = true;
				break;
		case -1:m_connectionError = ErrHostNotFound;
				m_connectionState = Disconnected;
				emit(connectionStateChanged());
				retVal = false;
				break;
		case -2:m_connectionError = ErrConnectionRefused;
				m_connectionState = Disconnected;
				emit(connectionStateChanged());
				retVal = false;
				break;
		case -3:m_connectionError = ErrTimeout;
				m_connectionState = Disconnected;
				emit(connectionStateChanged());
				retVal = false;
				break;
		default:m_connectionError = ErrUnknown;
				m_connectionState = Disconnected;
				emit(connectionStateChanged());
				retVal = false;
	}
	
	if (retVal==true)
	{	
		QObject::connect(m_telnet,SIGNAL(filledBuffer(QString)),m_mudstream,SLOT(processOutput(QString)));
		QObject::connect(m_telnet,SIGNAL(connectionClosed()),this,SLOT(closeConnection()));
		QObject::connect(m_view,SIGNAL(userInput(QString)),m_mudstream,SLOT(processInput(QString)));

		m_lastMudTitle = mudTitle;
		m_lastCharName = login;
		m_lastMudHost = mudHost;
		m_lastMudPort = mudPort;
	}

	return retVal;
}

/** This is used to reconnected to the last mud that was connected
  * @return true when connection attpemt was successful,
  *         returns false when not. in this case see connectionError()
  */
bool KmudCore::reconnect(void)
{
	return connectToMud(m_lastMudTitle,m_lastCharName,m_lastMudHost,m_lastMudPort);
}

/** Get the hostname of the last mud that was connected too */
QString KmudCore::getLastHost()
{
	return m_lastMudHost;
}

/** Get the port of the last mud that was connected too */
QString KmudCore::getLastPort()
{
	QString s;
	s.setNum(m_lastMudPort);
	return s;
}

void KmudCore::closeConnection()
{
	QObject::disconnect(m_telnet,SIGNAL(filledBuffer(QString)),m_mudstream,SLOT(processOutput(QString)));
	QObject::disconnect(m_telnet,SIGNAL(connectionClosed()),this,SLOT(closeConnection()));
	QObject::disconnect(m_view,SIGNAL(userInput(QString)),m_mudstream,SLOT(processInput(QString)));
	m_mudstream->processOutput("Connection closed.");

	if (m_telnet->isConnected())
		m_telnet->close();

	m_connectionState = Disconnected;
	emit(connectionStateChanged());
}

ConfigAPI* KmudCore::configAPI()
{
	return m_configManager->configAPI();
}

ProfileAPI* KmudCore::profileAPI()
{
	return m_configManager;
}

void KmudCore::setProfile (const QString& profile)
{
	QString oldMudProfile = m_mudProfile;
	QString oldCharProfile = m_charProfile;

	m_mudProfile = QString::null;
	m_charProfile = QString::null;

	if (!profile.isEmpty())
	{
		QStringList names = QStringList::split("/", profile);
		if (names.count() == 2)
		{
			m_mudProfile = names[0];
			m_charProfile = names[1];
		}
	}

	if (m_mudProfile != oldMudProfile || m_charProfile != oldCharProfile)
		emit profileChanged();
}

QString KmudCore::profile()
{
	if (!m_mudProfile.isEmpty() && !m_charProfile.isEmpty())
		return m_mudProfile+"/"+m_charProfile;
	else
		return QString::null;
}

QStringList KmudCore::listProfiles(const QString& baseProfile)
{
	return m_configManager->listProfiles(baseProfile);
}

KmudAPI::ConnectionError KmudCore::connectionError()
{
	return m_connectionError;
}

KmudAPI::ConnectionState KmudCore::connectionState()
{
	return m_connectionState;
}

KmudPart* KmudCore::loadPart(QString s)
{
	KTrader::OfferList offers;
	KService::Ptr svcptr=0L;
	KLibFactory* factory=0L;
	KmudPart* part=0L;

	kdDebug(DAREA_CORE) << "Requesting factory for service " << s << endl;
	offers = KTrader::self()->query(s);
	svcptr = offers.first();
	if (svcptr==0L)
	{
		kdDebug(DAREA_CORE) << "No offers for service " << s << endl;
		kdDebug(DAREA_CORE) << "This might be caused by an incomplete installation," << endl;
		kdDebug(DAREA_CORE) << "or by KDE not recognizing the new services." << endl;
/*		// same in a message box
		KMessageBox::detailedError(this,
		    i18n("Could not find a Kpart for mapping."),
		    i18n("KTrader was not able to find a Kpart for the servicetype Kmud/MapperPart.\n"
		    "Kmud installation should have installed a file called kmudmapper.desktop into\n"
		    "the 'services' directory of KDE.\n"
		    "If this file exists try running kbuildsycoca from a console to ensure\n"
		    "KDE recognizes the new service."));
*/
		factory=0L;

	}
	else
	{
		factory = KLibLoader::self()->factory(svcptr->library());

		if (factory==0L)
		{
			kdDebug(DAREA_CORE) << "Could not create a factory for KTrader's offer!" << endl;
			part=0L;
		}
		else
		{
			factory->setName("KmudFactory");
			kdDebug(DAREA_CORE) << "Got factory (" << factory << "). Creating KPart..." << endl;
			part=dynamic_cast<KmudPart*>(dynamic_cast<KmudFactory*>(factory)->createKmudPart(this,m_kmud,svcptr));

			if (part==0L)
			{
				kdDebug(DAREA_CORE) << "Factory could not create a KmudPart" << endl;
			
/*		// same in a message box
		QString details = i18n("Kmud was not able to load the KmudPart for mapping.\n");
		if (!factory)
		{
			details = i18n("The library (%1)"
			    "suggested for servicetype Kmud/MapperPart did not contain\n"
			    "a factory for KParts or KDE was not ablt to open it.\n").arg(ptr->library());
		}
		else
		{
			details = i18n("The factory of library %1 was not able to create a mapper.").arg(ptr->library());
		}
		KMessageBox::detailedError(this,
		    i18n("Unable to load the mapper."),
		    details);
*/

			}
			else
			{
				// the view manager is our partmanager. it will also call createGUI.
				kdDebug(DAREA_CORE) << "Adding plugin : " << part->name() << endl;
				m_partManager->addPart(part);
				m_partList.append(part);
				//m_kmud->createGUI(part);
				kdDebug(DAREA_CORE) << "Plugin Added : " << part->name() << endl;
			}

		}
	}

	return part;
}

void KmudCore::appendOutput(QString s)
{
	m_kmud->getView()->addText(s);
}

KStatusBar* KmudCore::mainStatusBar()
{
	return m_kmud->statusBar();
}

/** This is used to get a list of the parts */
QPtrList<KmudPart> *KmudCore::getPartsList(void)
{
	return &m_partList;
}

/** This will return a pointer to the global config settings */
KmudConfigCache *KmudCore::configCache()
{
	return m_configManager->configCache();
}

/**
 * This is used to send a command to the mud
 * @param command The command to send
 */
void KmudCore::sendCommand(QString command,bool echoOutput)
{
	// m_telnet->sendData(command);
	m_mudstream->processInput(command);
	if (echoOutput)
	{
		m_kmud->getView()->addEchoText(command);
	}
}

