/***************************************************************************
                               kmudcore.h
                             -------------------
    begin                : Sun May 20 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDCORE_H
#define KMUDCORE_H

#include <qptrlist.h>
#include <qstringlist.h>

#include <kmudapi.h>
#include <kmudfilter.h>

#include <kparts/partmanager.h>

class KmudMainWindow;
class KmudViewManager;
class KmudConfigManager;
class KmudView;
class CTelnet;
class CMudStream;
class KmudPart;
class KmudFilter;
class KmudConfigCache;
class CAutoLoginFilter;

/**
  * Kmud core class.
  * Implements the KmudAPI.
  * This class is intentionally _not_ singleton. We create a new core for every new main window.
  *
  *@author Kmud Developer Team
  */


class KmudCore : public KmudAPI
{
	Q_OBJECT

public:
	KmudCore();
	~KmudCore();
	
	/**
	  * connects to the given MUD using a text connection string
	  * @return ,  true when connection attpemt was successful
	  *         , false when not. in this case see connectionError()
	  */
	bool connectToMud(QString connectStr);

	/** This is used to reconnected to the last mud that was connected
	  * @return true when connection attpemt was successful,
      *         returns false when not. in this case see connectionError()
      */
	bool reconnect(void);

	KmudViewManager* viewManager();
	KmudMainWindow* mainWindow();
	CTelnet* telnetConnection();
	CMudStream* mudStream();

	ConfigAPI* configAPI();

	ProfileAPI* profileAPI();

	/**
	  * Set the profile (mud AND char profile)
	  */
	void setProfile (const QString& profile);
	/**
	  * Returns the currently active profile or QString::null.
	  */
	QString profile();
	
	/**
	  * Retrieves a list of profile belonging to a base profile.
	  * If the baseProfile string is empty it will return a list of mud profiles
	  * If it contains the name of a mud profile, it will return a list of character
      * profiles of this mud.
	  */
	QStringList listProfiles(const QString& baseProfile = QString::null);

	/**
	  * if there was an error during the last connection attempt, then
	  * this method will give you the kind of error
	  */
	KmudAPI::ConnectionError connectionError();
	/**
	  * returns the current state of the MUD connection
	  */
	KmudAPI::ConnectionState connectionState();

	/**
	  * loads a KmudPart of the given service type
	  */
	KmudPart* loadPart(QString);

	/**
	  * aappends text to the output view
	  */
	void appendOutput(QString);

	/**
	  * returns the statusbar of the mainwindow
	  */
	KStatusBar* mainStatusBar();

	/** Get the hostname of the last mud that was connected too */
	QString getLastHost();
	/** Get the port of the last mud that was connected too */
	QString getLastPort();
	/** This is used to get a list of the parts */
	QPtrList<KmudPart> *getPartsList(void);

	/** This will return a pointer to the global config settings */
	KmudConfigCache *configCache();

	/**
	 * This is used to send a command to the mud
	 * @param command The command to send
	 */
	void sendCommand(QString command,bool echoOutput);

	
public slots:
	/** closes the MUD connection */
	void closeConnection();

private:
	/** The last mud that was connected too */
	QString m_lastMudTitle;
	/** The last character name that was used */
	QString m_lastCharName;
	/** The last mud hostname that was connected too */
	QString m_lastMudHost;
	/** The last mud port that was used in the last connection */
	uint m_lastMudPort;

	/** the main window of Kmud */
	KmudMainWindow* m_kmud;

	/** the view manager */
	KmudViewManager* m_viewManager;

	/** the view manager */
	KParts::PartManager* m_partManager;

	/** the list of all parts loaded */
	QPtrList<KmudPart>	m_partList;
	
	/** the config manager */
	KmudConfigManager* m_configManager;

	/** the main view */
	KmudView* m_view;
	
	/** the telnet connection */
	CTelnet* m_telnet;

	/** the mud stream */
	CMudStream* m_mudstream;

	KmudAPI::ConnectionError m_connectionError;
	KmudAPI::ConnectionState m_connectionState;


	/** Current MUD and character profile */
	QString m_mudProfile;
	QString m_charProfile;	

	KmudFilter* m_aliasFilter;

	CAutoLoginFilter* m_autoLoginFilter;

private:
	/**
	  * connects to the given MUD.
	  * @return ,  true when connection attpemt was successful
	  *         , false when not. in this case see connectionError()
	  */
	bool connectToMud(QString mudTitle, QString login, QString mudHost, uint mudPort);
};

#endif

