/***************************************************************************
                               kmudview.h
                             -------------------
    begin                : Sat Mar  3 19:17:48 CET 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KMUDVIEW_H
#define KMUDVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

// include files for Qt
#include <qwidget.h>
#include <qlayout.h>
#include <qpopupmenu.h>

// include files for KDE
#include <kcombobox.h>

// includes for kmud
#include <kmudconfigcache.h>

// forward declarations
class KmudFilter;
class KmudViewFilter;

class CMudView;

/** The KmudView class provides the view widget for the KmudApp instance.	
 * The View instance inherits QWidget as a base class and represents the view object of a KTMainWindow. As KmudView is part of the
 * docuement-view model, it needs a reference to the document object connected with it by the KmudApp class to manipulate and display
 * the document structure provided by the KmudDoc class.
 * 	
 * @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
 * @version KDevelop version 0.4 code generation
 */
class KmudView : public QWidget
{
	Q_OBJECT
public:
	/** Constructor for the main view */
	KmudView(QWidget *parent = 0, const char *name=0);

	/** Destructor for the main view */
	~KmudView();

	/** Set the defaults for Font, text color and background color.  */
	void init(const QFont font, const QColor textColor, const QColor bgColor);

	/** Shows the Kmud splash document, i.e. app_dir/kmud/splash/splash.html */
	void splash();

	/**
	  * Sets the font for all following addText operations until the next setFont() or clear() call.
	  * Status of the style setting (bold, underline, italic) of the QFont object will be ignored
	  * if useStyle is false.
	  */
	void setFont(QFont font, bool useStyle = false);

	/** Returns the currently used font, including the style settings.  */
	QFont font();

	/** Sets the underline style on if underline is true */
	void setUnderline(bool underline);

	/** Returns true if underline is active */
	bool underline();

	/** Sets the italic style on if italic is true */
	void setItalic(bool italic);

	/** Returns true if italic is active */
	bool italic();

	/** Sets the bold style on if bold is true */
	void setBold(bool bold);

	/** Returns true if bold is active */
	bool bold();

	/** Sets the text color. */
	void setTextColor(QColor color);

	/** Returns the currently used text color. */
	QColor textColor();

	/** Sets the text background color. */
	void setBgColor(QColor color);

	/** Returns the currently used text background color. */
	QColor bgColor();
	
	/** Sets the following text blinking */
	void setBlink(bool state);
	/** Returns the blink state for following text */
	bool blink();
	
	/**
	  * If enable is true, the view will update everytime addText is called.
	  * This might be undesired when addText is called more than once per line.
	  * Default is true;
	  */
	void setUpdatesEnabled(bool enable);

	/**
	  * Returns true when the view's content will be updated on every addText call.
	  * This can be set by the setUpdatesEnabled(bool) method.
	  */
	bool updatesEnabled();

	/**
	  * This method is used to set the input line font
	  */
	void setInputFont(QFont font);

	/**
	  * This method is used to set the local echo mode
	  */
	void setLocalEchoMode(KmudConfigCache::EchoState mode);

	/**
	  * This method is used to set the color of the local echo
	  */
	void setLocalEchoColor(QColor color);

	/** This method is used to set the input echo mode */
	void setInputEchoMode(KmudConfigCache::EchoState mode);

	/** This method is used to set if the input should be kept when return is pressed */
	void setKeepLastInput(bool keep);

	/** This is used to suppress new lines when echo is off */
	void setSuppressNewLine(bool suppress);
	/** This method is used to set if the input should be selected after pressig return */
	void setSelectLastInput(bool select);
	/** This method is used to set if dual input lines are to be used */
	void setDualInputLines(bool dual);

public slots:
	/** This is used to set if the input line should display text entered into it */
	void setInputEcho(bool echo);
	
	/** This method is when echo is turned on or off by the CTelnet class which will
	  * recived the commands from the mud
	  */
	void setLocalEcho(bool echo);

	/**
	  * Clears the view and resets font and color settings to the default values.
	  * If no default values were set, the font defaults to the QFont default, text color to red
	  * and background color to black.
	  */
	void clear();
	
	/**
	  * Adds text to the view using the current settings for font, fontstyles (bold, underline, italic),
	  * text color and text background color
	  */
	void addText(QString text);

	/**
	 * This is used to add text to the mud view in the local echo color
	 * @param text The text to add
	 */
	void addEchoText(QString text);

	/**
	  * Returns the output filter.
	  * The filter parses the output from the mud, interprets ANSI sequences and controls the
	  * view accordingly.
	  */
	KmudFilter* filter() const;	

private:
	/** This is true or false depedning in the mud as requested echo to be on or off */
	bool m_localEcho;
	/** If this is false then the input line will not display text entered into it  */
	bool m_inputEcho;
	/** Used to store the local echo color */
	QColor m_localEchoColor;
	/** Used to store the local echo mode */
	KmudConfigCache::EchoState m_localEchoMode;
	/** Used to store the input echo mode */
	KmudConfigCache::EchoState m_inputEchoMode;
	/** Used to store if the input line should keep it's contents when return is pressed */
	bool m_keepLastInput;
	/** Used to store if the input line should be selected after pressing return */
	bool m_selectLastInput;
	/** Used to store if there are to be dual input lines */
	bool m_dualInput;
	/** Used to store if new lines should be supressed */
	bool m_suppressNewLine;

	/** The layout of the mud view */
	QVBoxLayout* m_layout;
	/** The output view of the mud */
	CMudView* m_outputView;
	/** The first input line */
	KHistoryCombo* m_input1;
	/** The second input line which is visible when m_dualInput is true */
	KHistoryCombo* m_input2;
	KmudViewFilter* m_filter;

private slots: // Private slots
	void slotInputReturnPressed(KHistoryCombo *lineEdit);
	void slotInputReturnPressed1();
	void slotInputReturnPressed2();
	void slotInputActivated1(const QString& s);
	void slotInputActivated2(const QString& s);


signals:
	void userInput(QString input);

};

#endif // KMUDVIEW_H

