/***************************************************************************
                          kcmapfile.cpp
                      -------------------
    description          :
    begin                : Mon Jan 1 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/


#include "kcmapfile.h"

#include "../cmaproom.h"
#include "../cmapzone.h"
#include "../cmappath.h"
#include "../cmaptext.h"

#include <kmuddebug.h>

KCMapFile::KCMapFile(QString filename,int gridWidth,int gridHeight):KConfig(filename)
{
	setDollarExpansion(false);

	m_gridWidth = gridWidth;
	m_gridHeight = gridHeight;
}

KCMapFile::~KCMapFile()
{
}

void KCMapFile::setRoomGroup(int rnum)
{
	setGroup(numberedGroup("ROOM-", rnum));
}

void KCMapFile::setPathGroup(int pnum)
{
	setGroup(numberedGroup("PATH-", pnum));
}

void KCMapFile::setTextGroup(int tnum)
{
	setGroup(numberedGroup("TEXT-", tnum));
}

void KCMapFile::setLevelGroup(int lnum)
{
	setGroup(numberedGroup("LEVEL-", lnum));
}

void KCMapFile::setZoneGroup(int znum)
{
	setGroup(numberedGroup("ZONE-", znum));
}

void KCMapFile::setSpeedwalkGroup(int snum)
{
	setGroup(numberedGroup("SPEEDWALK-", snum));
}

QString KCMapFile::numberedGroup(QString groupPrefix, int number)
{
	QString numString;
	numString.setNum(number);
	groupPrefix.append(numString);

	return groupPrefix;
}

void KCMapFile::writeVersion(int major, int minor)
{
	const char* lastGroup = group();
	setGroup("VERSION");
	writeEntry("major", major);
	writeEntry("minor", minor);
	setGroup(lastGroup);
}

void KCMapFile::readVersion(int &major, int &minor)
{
	const char* lastGroup = group();
	setGroup("VERSION");
	major = readNumEntry("major", 0);
	minor = readNumEntry("minor", 0);
	setGroup(lastGroup);
}

void KCMapFile::writeCount(QString name, int number)
{
	const char* lastGroup = group();
	setGroup("COUNTS");
	writeEntry(name, number);
	setGroup(lastGroup);
}

int KCMapFile::readCount(QString name)
{
	int result;
	const char* lastGroup = group();
	setGroup("COUNTS");
	result = readNumEntry(name, 0);
	setGroup(lastGroup);

	return result;
}

void KCMapFile::loadZone(CMapZone *zone)
{
	// mapLevel = kcmfile->readNumEntry("level", 0);
	zone->setLowPos(QPoint(readNumEntry("X", -1) * m_gridWidth,readNumEntry("Y", -1)* m_gridHeight));
	zone->setLabel(readEntry("name", ""));
}

void KCMapFile::loadRoom(CMapRoom *room)
{
	// mapLevel = kcmfile->readNumEntry("level", 0);
    room->setUseDefaultCol(readBoolEntry("use default color", true));
	room->setColor(readColorEntry("color"));
	room->setLowPos(QPoint(readNumEntry("X", -1) * m_gridWidth,readNumEntry("Y", -1)* m_gridHeight));
	room->setLabel(readEntry("label", ""));
	room->setDescription(readEntry("description", ""));

	
}

void KCMapFile::loadText(CMapText *text)
{
	// lvl = kcmfile->readNumEntry("level", 0);
	QColor* defCol = new QColor(text->getColor());

	text->setLowPos(QPoint(readNumEntry("X", -1) * m_gridWidth,readNumEntry("Y", -1)* m_gridHeight));

	text->setFont(readFontEntry("font"));
	text->setColor(readColorEntry("color", defCol));
	delete defCol;

	text->setText(readEntry("text"));
}

void KCMapFile::loadPath(CMapPath *path)
{
	path->setSpecialExit(readBoolEntry("special exit", false));
	path->setSpecialCmd(readEntry("special command", ""));

	// Load the path bends
	QString bendnum;
	QString bendpre("bend-");
	int num_bends = readNumEntry("bends", 0);

	for (int count = 0; count < num_bends; count++)
	{
		bendnum.setNum(count+1);
		path->addBendWithUndo(readPointEntry(bendpre+bendnum));
	}
}

