/***************************************************************************
                               cmapviewmovetracker.cpp
                             -------------------
    begin                : Fri May 11 2001
    copyright            : (C) 2001 by Kmud Developer Team
    email                : kmud-devel@kmud.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cmapviewmovetracker.h"

#include <qwidget.h>
#include <qevent.h>

#include "../../cmapmanager.h"
#include "../../cmaplevel.h"

#include <kmuddebug.h>

CMapViewMoveTracker::CMapViewMoveTracker(CMapManager *manager,QWidget *parent, const char *name ) : CMapViewBase(manager,parent,name)
{
	m_buffer = NULL;
	setBackgroundMode(NoBackground);

	m_needsCreate = true;

	m_pos = QPoint (0,0);
}

CMapViewMoveTracker::~CMapViewMoveTracker()
{
	if (m_buffer)
		delete m_buffer;
}

/** This method is called when an element is added */
void CMapViewMoveTracker::addedElement(CMapElement *element)
{
	if (isElementVisibale(element))
	{
		redraw();
	}
}

/** This method is called when an element is deleted */
void CMapViewMoveTracker::deletedElement(CMapLevel *deletedFromLevel)
{
	QPaintEvent tmpEvent();

	CMapLevel *upperLevel = getCurrentlyViewedLevel()->getNextLevel();
	CMapLevel *lowerLevel = getCurrentlyViewedLevel()->getPrevLevel();


	if (deletedFromLevel == getCurrentlyViewedLevel())
	{
		redraw();
	}

	if (upperLevel  && mapManager->getMapData()->showUpperLevel)
		if (deletedFromLevel == upperLevel)
			redraw();

	if (lowerLevel  && mapManager->getMapData()->showLowerLevel)
		if (deletedFromLevel == lowerLevel)
			redraw();
}

/** This method is called when an element is changed */
void CMapViewMoveTracker::changedElement(CMapElement *element)
{
	if (isElementVisibale(element))
		redraw();
}

/** This method is called when a map level is changed */
void CMapViewMoveTracker::changedLevel(CMapLevel *level)
{
	if (isLevelVisibale(level))
	{
		redraw();
	}
}

/** Tell this map widget to display a different zone */
void CMapViewMoveTracker::showPosition(QPoint pos,CMapLevel *level,bool)
{
	setLevel(level);

	m_pos = pos;

	redraw();
}

/** Used to find the width of the widget */
int CMapViewMoveTracker::getWidth(void)
{
	return width();
}

/** Used to find the height of the widget */
int CMapViewMoveTracker::getHeight(void)
{
	return height();
}

/** Used to set the follow mode */
void CMapViewMoveTracker::setFollowMode(bool)
{
	// DO NOTHING
}

/** Get the state of the follow combo box */
bool CMapViewMoveTracker::getFollowMode(void)
{
	return true;
}

/** Used to let the map manager know if it should register the focus of this widget */
bool CMapViewMoveTracker::acceptFocus(void)
{
	return false;
}

/** Used to set the view to active */
void CMapViewMoveTracker::setActive(bool)
{
	// DO NOTHING
}

/** This is used to generate the conents of the view */
void CMapViewMoveTracker::generateContents(void)
{	
	QRect drawArea(0,0,width(), height());

	// Create double buffer
	if (m_buffer && (m_buffer->size() != drawArea.size()))
	{
		delete m_buffer;
		m_buffer = NULL;
	}
	
	if (!m_buffer)
	{
		m_buffer = new QPixmap(drawArea.size());
	}

	if (m_needsCreate)
	{
		QPainter p;
    	
		p.begin(m_buffer);

		if (getCurrentlyViewedZone()->getUseDefaultBackground())
		{
			p.fillRect(drawArea,mapManager->getMapData()->backgroundColor);
		}
		else
		{
			p.fillRect(drawArea,getCurrentlyViewedZone()->getBackgroundColor());
		}

		int offset_x = m_pos.x()-(width()/2);
		int offset_y = m_pos.y()-(height()/2);
		p.translate(-offset_x,-offset_y);

		// drawGrid(&p);
		drawElements(&p);

		p.end();

		m_needsCreate = false;
	}
}


void CMapViewMoveTracker::paintEvent(QPaintEvent *)
{
	generateContents();

	bitBlt(this, 0, 0, m_buffer);
}

void CMapViewMoveTracker::redraw(void)
{
	update();
	m_needsCreate = true;
}

void CMapViewMoveTracker::resizeEvent(QResizeEvent *)
{
	m_needsCreate = true;
}

/** This method is called when the current position of the player changes */
void CMapViewMoveTracker::playerPositionChanged(CMapRoom *room)
{
	m_pos = QPoint(room->getX(),room->getY());
	redraw();
}
